@@ -1,12 +1,11 @@
 utils::globalVariables(c(
-  ".GRP", "distance", "dups", "from", "i.size", "ind", "indClDT", "initialPixels",
-  "keep",
+  ".GRP", "distance", "dups", "from", "i.size", "ind", "indClDT", "initialPixels", "keep",
   "n", "newQuantity", "numNeighs", "numRetries", "origIndex", "pixels", "proportion",
   "quantityAdj", "quantityAdj2", "state", "size", "tooBigByNCells", "V1"
 ))
 
 ################################################################################
-#' Simulate a contagious spread process on a landscape, with data.table internals
+#' Simulate a contagious spread process on a landscape, with `data.table` internals
 #'
 #' This can be used to simulate fires, seed dispersal, calculation of iterative,
 #' concentric, symmetric (currently) landscape values and many other things.
@@ -21,17 +20,17 @@ utils::globalVariables(c(
 #' Using `spreadProb`, every "active" pixel will assess all
 #' neighbours (either 4 or 8, depending on  `directions`), and will "activate"
 #' whichever neighbours successfully pass independent calls to
-#' `runif(1,0,1)<spreadProb`.
+#' `runif(1,0,1) < spreadProb`.
 #' The algorithm will iterate again and again, each time starting from the newly
 #' "activated" cells. Several built-in decisions are as follows.
-#' 1. no active cell can active a cell that was already activated by
+#' 1. no active cell can activate a cell that was already activated by
 #' the same event (i.e., "it won't go backwards"). 2. If `allowOverlap` is
 #' `FALSE`, then the previous rule will also apply, regardless of which
 #' "event" caused the pixels to be previously active.
 #'
 #' This function can be interrupted before all active cells are exhausted if
 #' the `iterations` value is reached before there are no more active
-#' cells to `spread2` into. The interrupted output (a data.table) can be passed
+#' cells to `spread2` into. The interrupted output (a `data.table`) can be passed
 #' subsequently as an input to this same function (as `start`).
 #' This is intended to be used for situations where external events happen during
 #' a spread2 event, or where one or more arguments to the spread2 function
@@ -52,16 +51,16 @@ utils::globalVariables(c(
 #' \deqn{max(spreadProb)/min(spreadProb)} will generally be less than
 #' `asymmetry`, for the 8 neighbours. The exact adjustment to the spreadProb
 #' is calculated with:
-#' \deqn{angleQuality <- (cos(angles - rad(asymmetryAngle))+1)/2}
+#' \deqn{angleQuality <- (cos(angles - CircStats::rad(asymmetryAngle))+1)/2}
 #' which is multiplied to get an angle-adjusted spreadProb:
 #' \deqn{spreadProbAdj <- actualSpreadProb * angleQuality}
 #' which is then rescaled:
 #' \deqn{adjustedSpreadProb = (spreadProbAdj - min(spreadProbAdj)) * par2 + par1},
 #' where par1 and par2 are parameters calculated internally to make the 2 conditions above true.
 #'
-#' @section Breaking out of spread2 events:
+#' @section Breaking out of `spread2` events:
 #'
-#' There are 3 ways for the spread2 to "stop" spreading.
+#' There are 3 ways for the `spread2` to "stop" spreading.
 #' Here, each "event" is defined as all cells that are spawned from each unique
 #' `start` location.
 #' The ways outlined below are all acting at all times, i.e., they are not
@@ -70,10 +69,10 @@ utils::globalVariables(c(
 #' are interacting with each other correctly.
 #'
 #' \tabular{ll}{
-#'   `spreadProb` \tab Probabilistically, if spreadProb is low enough,
-#'                          active spreading events will stop. In practice,
-#'                          this number generally should be below 0.3 to actually
-#'                          see an event stop\cr
+#'   `spreadProb` \tab Probabilistically, if `spreadProb` is low enough,
+#'                          active spreading events will stop.
+#'                          In practice, this number generally should be below 0.3
+#'                          to actually see an event stop.\cr
 #'   `maxSize` \tab This is the number of cells that are "successfully" turned
 #'                       on during a spreading event. `spreadProb` will still
 #'                       be active, so, it is possible that the end size of each event
@@ -95,10 +94,10 @@ utils::globalVariables(c(
 #'                  locations for spreading events to start and spread2 into. Required.
 #'
 #' @param start Required. Either a vector of pixel numbers to initiate spreading, or a
-#'              data.table that is the output of a previous `spread2`.
+#'              `data.table` that is the output of a previous `spread2`.
 #'              If a vector, they should be cell indices (pixels) on the `landscape`.
 #'              If user has x and y coordinates, these can be converted with
-#'              [`cellFromXY()`][raster::cellFromXY].
+#'              [`cellFromXY()`][terra::cellFromXY].
 #'
 #' @param spreadProb  Numeric of length 1 or length `ncell(landscape)` or
 #'                    a `RasterLayer` that is the identical dimensions as
@@ -140,11 +139,10 @@ utils::globalVariables(c(
 #'                 where raster non NA values indicate the cells that were "active", and the
 #'                 value is the initial starting pixel.
 #'
-#' @param maxSize       Numeric. Maximum number of cells for a single or
-#'                      all events to be spread2. Recycled to match `start` length,
-#'                      if it is not as long as `start`. This will be overridden if
-#'                      `exactSize` also provided.
-#'                      See section on `Breaking out of spread2 events`.
+#' @param maxSize  Numeric. Maximum number of cells for a single or all events to be `spread2`.
+#'                 Recycled to match `start` length, if it is not as long as `start`.
+#'                 This will be overridden if `exactSize` also provided.
+#'                 See section on 'Breaking out of `spread2` events'.
 #'
 #' @param exactSize Numeric vector, length 1 or `length(start)`.
 #'                  Similar to `maxSize`, but these will be the exact
@@ -156,7 +154,7 @@ utils::globalVariables(c(
 #' @param directions    The number adjacent cells in which to look;
 #'                      default is 8 (Queen case). Can only be 4 or 8.
 #'
-#' @param iterations    Number of iterations to spread2.
+#' @param iterations    Number of iterations to `spread2`.
 #'                      Leaving this `NULL` allows the spread2 to continue
 #'                      until stops spreading itself (i.e., exhausts itself).
 #'
@@ -183,14 +181,14 @@ utils::globalVariables(c(
 #'              This is likely most useful in repeated iteration cases i.e., if this call
 #'              is using the previous output from this same function.
 #'
-#' @param neighProbs An optional numeric vector, whose sum is 1. It indicates the
-#'                   probabilities that an individual
-#'                   spread iteration will spread to `1, 2, ..., length(neighProbs)`
-#'                   neighbours, respectively. If this is used (i.e., something other than
-#'                   NA), `circle` and `returnDistances` will not work currently.
+#' @param neighProbs An optional numeric vector, whose sum is 1.
+#'                   It indicates the probabilities that an individual spread iteration
+#'                   will spread to `1, 2, ..., length(neighProbs)` neighbours, respectively.
+#'                   If this is used (i.e., something other than `NA`), `circle` and
+#'                   `returnDistances` will not work currently.
 #' @param maxRetriesPerID Only active if `exactSize` is used. This is the number of attempts
 #'                        that will be made per event ID, before abandoning, therefore completing
-#'                        the spread2 for that event with a size that is smaller than
+#'                        the `spread2` for that event with a size that is smaller than
 #'                        `exactSize`. Default 10 times.
 #'
 #' @param asymmetry     A numeric or `RasterLayer` indicating the ratio of the
@@ -218,20 +216,20 @@ utils::globalVariables(c(
 #'
 #' @details
 #'
-#' If `exactSize` or `maxSize` are used, then spreading will continue and stop
-#' before or at `maxSize` or at `exactSize`. If `iterations` is specified,
-#' then the function will end, and the returned `data.table` will still
+#' If `maxSize` or `exactSize` are used, then spreading will continue and stop
+#' before or at `maxSize` or at `exactSize`, respectively.
+#' If `iterations` is specified, then the function will end, and the returned `data.table`
 #' may (if `maxSize`) or will (if `exactSize`) have at least one active
-#' cell per event that did not already achieve `maxSize` or `exactSize`. This
-#' will be very useful to build new, customized higher-level wrapper functions that iteratively
-#' call `spread2`.
+#' cell per event that did not already achieve `maxSize` or `exactSize`.
+#' This will be very useful to build new, customized higher-level wrapper functions that
+#' iteratively call `spread2`.
 #'
 #' @note
 #' `exactSize` may not be achieved if there aren't enough cells in the map.
 #' Also, `exactSize` may not be achieved because the active cells are "stuck",
-#' i.e., they have no unactivated cells to move to; or the `spreadProb` is low.
+#' i.e., they have no inactivated cells to move to; or the `spreadProb` is low.
 #' In the latter two cases, the algorithm will retry again, but it will only
-#' re-try from the last iterations active cells.
+#' retry from the last iteration's active cells.
 #' The algorithm will only retry 10 times before quitting.
 #' Currently, there will also be an attempt to "jump" up to four cells away from
 #' the active cells to try to continue spreading.
@@ -243,7 +241,7 @@ utils::globalVariables(c(
 #'
 #' This function can be used iteratively, with relatively little overhead compared to using
 #' it non-iteratively. In general, this function can be called with arguments set as user
-#' needs, and with specifying iterations = 1 (say). This means that the function will spread
+#' needs, and with specifying e.g., `iterations = 1`. This means that the function will spread
 #' outwards 1 iteration, then stop. The returned object will be a `data.table` or
 #' `RasterLayer` that can be passed immediately back as the start argument into a subsequent
 #' call to `spread2`. This means that every argument can be updated at each iteration.
@@ -257,7 +255,7 @@ utils::globalVariables(c(
 #' objects by their pixel location, increasing.
 #' Then, of course, sorting any vectorized arguments (e.g., `maxSize`) accordingly.
 #'
-#' **NOTE**: the `data.table` or `RasterLayer` should not use be altered
+#' **NOTE**: the `data.table` or `RasterLayer` should not be altered
 #' when passed back into `spread2`.
 #'
 #' @section `allowOverlap`:
@@ -288,7 +286,7 @@ utils::globalVariables(c(
 #' attribute) will be attached to the `Raster` as an attribute named `pixel` as it
 #' provides pixel-level information about the spread events.
 #'
-#' The `RasterLayer` represents every cell in which a successful spread2 event occurred.
+#' The `RasterLayer` represents every cell in which a successful `spread2` event occurred.
 #' For the case of, say, a fire this would represent every cell that burned.
 #' If `allowOverlap` is `TRUE`, the return will always be a `data.table`.
 #'
@@ -297,13 +295,13 @@ utils::globalVariables(c(
 #'
 #' \tabular{ll}{
 #'   `initialPixels` \tab the initial cell number of that particular
-#'                            spread2 event.\cr
+#'                            `spread2` event.\cr
 #'   `pixels` \tab The cell indices of cells that have
-#'                        been touched by the spread2 algorithm.\cr
+#'                        been touched by the `spread2` algorithm.\cr
 #'   `state` \tab a logical indicating whether the cell is active (i.e.,
 #'                        could still be a source for spreading) or not (no
 #'                        spreading will occur from these cells).\cr
-#'   `from` \tab The pixel indices that were the immediately preceeding
+#'   `from` \tab The pixel indices that were the immediately preceding
 #'                    "source" for each `pixels`, i.e., the lag 1 pixels.
 #'                    Only returned if `returnFrom` is `TRUE` \cr
 #' }
@@ -313,7 +311,7 @@ utils::globalVariables(c(
 #' \tabular{ll}{
 #'   `id` \tab An arbitrary code, from 1 to `length(start)` for each "event".\cr
 #'   `initialPixels` \tab the initial cell number of that particular
-#'                            spread2 event.\cr
+#'                            `spread2` event.\cr
 #'   `numRetries` \tab The number of re-starts the event did because it got
 #'                          stuck (normally only because `exactSize` was used
 #'                          and was not achieved.\cr
@@ -322,7 +320,7 @@ utils::globalVariables(c(
 #'   `size` \tab The current size, in pixels, of each event.\cr
 #' }
 #' and several other objects that provide significant speed ups in iterative calls to
-#' spread2. If the user runs `spread2` iteratively, there will likely be significant
+#' `spread2`. If the user runs `spread2` iteratively, there will likely be significant
 #' speed gains if the `data.table` passed in to `start` should have the attribute
 #' attached, or re-attached if it was lost, e.g., via
 #' `setattr(outInput, "spreadState", attr(out, "spreadState"))`, where `out` is the
@@ -332,16 +330,13 @@ utils::globalVariables(c(
 #'
 #' @author Eliot McIntire and Steve Cumming
 #' @export
-#' @importFrom checkmate assert assertClass assertNumeric
+#' @importFrom checkmate assert assertClass assertMultiClass assertNumeric
 #' @importFrom checkmate checkClass checkDataTable checkLogical checkNumeric checkScalarNA
-#' @importFrom checkmate qassert
+#' @importFrom checkmate qassert checkMultiClass
 #' @importFrom data.table := alloc.col as.data.table copy data.table is.data.table
 #' @importFrom data.table rbindlist set setattr setcolorder setkeyv setnames uniqueN
-#' @importFrom fastmatch fmatch
 #' @importFrom fpCompare %<=% %>>%
-#' @importFrom magrittr %>%
-#' @importFrom quickPlot Plot
-#' @importFrom raster fromDisk ncell raster res ncol pointDistance
+#' @importFrom terra ncell res ncol distance
 #' @importFrom stats runif
 #'
 #' @seealso [spread()] for a different implementation of the same algorithm.
@@ -360,45 +355,52 @@ spread2 <- function(landscape, start = ncell(landscape) / 2 - ncol(landscape) /
                     skipChecks = FALSE) {
 
   #### assertions ###############
-  assertClass(landscape, "Raster")
+  checkmate::assertMultiClass(landscape, c("Raster", "SpatRaster"))
+  fmatch2 <- if (requireNamespace("fastmatch", quietly = TRUE)) fastmatch::fmatch else base::match
+  landscapeOrigClass <- is(landscape)
   ncells <- ncell(landscape)
   numCols <- ncol(landscape)
   anyNAneighProbs <- any(is.na(neighProbs))
   if (!skipChecks) {
     assert(
       checkNumeric(start, min.len = 0, max.len = ncells, lower = 1, upper = ncells),
-      checkClass(start, "Raster"),
+      checkMultiClass(start, c("Raster", "SpatRaster")),
       checkDataTable(start))
 
     qassert(neighProbs, "n[0,1]")
     assertNumeric(sum(neighProbs), lower = 1, upper = 1)
 
-    assert(
-      checkNumeric(spreadProb, 0, 1, min.len = ncell(landscape), max.len = ncell(landscape)),
+    # if (!inherits(spreadProb, "Raster") && !inherits(spreadProb, "SpatRaster")) {
+    assert(# this is "or"
+      checkNumeric(spreadProb, 0, 1, min.len = 1, max.len = ncell(landscape)),
       checkNumeric(spreadProb, 0, 1, min.len = 1, max.len = 1),
-      checkClass(spreadProb, "RasterLayer")
+      checkClass(spreadProb, "Raster"),
+      checkClass(spreadProb, "SpatRaster")
     )
+    # }
 
-    if (is(spreadProb, "Raster")) {
-      if (fromDisk(spreadProb)) {
+    if (is(spreadProb, "Raster") || is(spreadProb, "SpatRaster")) {
+      if (!terra::inMemory(spreadProb)) {
         warning("spreadProb is a raster layer stored on disk. This may cause spread2 to be",
                 " very slow. We suggest extracting the values to a numeric vector first, ",
                 "then passing this to spreadProb")
       }
     }
-    assert(checkNumeric(persistProb, 0, 1, min.len = 1, max.len = 1),
-           checkClass(persistProb, "RasterLayer"))
     assert(
-      checkScalarNA(spreadProbRel),
-      checkClass(spreadProbRel, "RasterLayer")
+      checkNumeric(persistProb, 0, 1, min.len = 1, max.len = 1),
+      checkMultiClass(persistProb,  c("RasterLayer", "SpatRaster"))
+    )
+    assert(
+      checkMultiClass(spreadProbRel, c("RasterLayer", "SpatRaster")),
+      checkScalarNA(spreadProbRel) ## needs to be checked second; will fail if SpatRaster
     )
     assert(
       checkNumeric(asymmetry, 0, Inf, min.len = 1, max.len = 1),
-      checkClass(asymmetry, "RasterLayer")
+      checkMultiClass(asymmetry, c("RasterLayer", "SpatRaster"))
     )
     assert(
       checkNumeric(asymmetryAngle, 0, 360, min.len = 1, max.len = 1),
-      checkClass(asymmetryAngle, "RasterLayer")
+      checkMultiClass(asymmetryAngle,  c("RasterLayer", "SpatRaster"))
     )
     qassert(directions, "N1[4,8]")
     qassert(iterations, "N1[0,Inf]")
@@ -452,12 +454,15 @@ spread2 <- function(landscape, start = ncell(landscape) / 2 - ncol(landscape) /
   # returnDistances = TRUE and circle = TRUE both require distance calculations
   needDistance <- returnDistances | circle | returnDirections
   usingAsymmetry <- !is.na(asymmetry)
+  asymmetryAngleNeedSubset <- (inherits(asymmetryAngle, "Raster") ||
+                                 inherits(asymmetryAngle, "SpatRaster")) &&
+    NROW(asymmetryAngle) != 1 # length was previously used, but has different meaning for SpatRaster & Raster
 
   # This means that if an event can not spread any more, it will try 10 times, incl. 2 jumps
   # maxRetriesPerID <- 10
 
   if (!is.numeric(start) & !is.data.table(start)) {
-    if (is(start, "Raster")) {
+    if (is(start, "Raster") || is(start, "SpatRaster")) {
       start <- attr(start, "pixel")
     } else {
       stop("Start must be either a vector of pixels, a data.table from",
@@ -469,10 +474,11 @@ spread2 <- function(landscape, start = ncell(landscape) / 2 - ncol(landscape) /
     # A "new" entry into spread2 -- need to set up stuff
     if (canUseAvailable) {
       #if (smallRaster) {
-      notAvailable <- if (requireNamespace("bit", quietly = TRUE))
+      notAvailable <- if (requireNamespace("bit", quietly = TRUE)) {
         bit::bit(ncells)
-      else
+      } else {
         logical(ncells)
+      }
       #} else {
       #  notAvailable <- ff(vmode = "boolean", FALSE, length = ncells)
       #}
@@ -606,7 +612,7 @@ spread2 <- function(landscape, start = ncell(landscape) / 2 - ncol(landscape) /
                              includeBehavior = "excludePixels",
                              minRadius = resCur,
                              maxRadius = 20 * resCur)[, "indices"]) # 20 pixels
-        }) %>%
+        }) |>
           do.call(what = rbind)
 
         dtPotential <- matrix(as.integer(dtPotential), ncol = 2)
@@ -668,7 +674,7 @@ spread2 <- function(landscape, start = ncell(landscape) / 2 - ncol(landscape) /
     if (needDistance) {
       fromPts <- xyFromCell(landscape, dtPotential$id)
       toPts <- xyFromCell(landscape, dtPotential$to)
-      dists <- pointDistance(p1 = fromPts, p2 = toPts, lonlat = FALSE)
+      dists <- terra::distance(fromPts, toPts, pairwise=TRUE, lonlat = FALSE)
       if (isTRUE(returnDirections))
         dirs <- .pointDirection(fromPts, toPts)
       if (usingAsymmetry) {
@@ -677,7 +683,7 @@ spread2 <- function(landscape, start = ncell(landscape) / 2 - ncol(landscape) /
         } else {
           asymmetry[dtPotential$to]
         }
-        actualAsymmetryAngle <- if (length(asymmetryAngle) == 1) {
+        actualAsymmetryAngle <- if (!asymmetryAngleNeedSubset) {
           asymmetryAngle
         } else {
           asymmetryAngle[dtPotential$to]
@@ -714,7 +720,6 @@ spread2 <- function(landscape, start = ncell(landscape) / 2 - ncol(landscape) /
         pureCircle <- cbind(pureCircle[, c("id", "indices", "dists"), drop = FALSE],
                             distClass = ceiling(pureCircle[, "dists"]))
         colnames(pureCircle)[2] <- c("to")
-
         theoreticalAngleQualities <- angleQuality(pureCircle[, "id", drop = FALSE],
                                                   pureCircle[, "to", drop = FALSE],
                                                   landscape,
@@ -817,7 +822,7 @@ spread2 <- function(landscape, start = ncell(landscape) / 2 - ncol(landscape) /
       setkeyv(dtPotential, c("id", "from")) # sort so it is the same as numNeighsByPixel
 
       if (NROW(dtPotential)) {
-        if (is(spreadProbRel, "RasterLayer")) {
+        if (is(spreadProbRel, "RasterLayer") || is(spreadProbRel, "SpatRaster")) {
           set(dtPotential, NULL, "spreadProbRel", spreadProbRel[][dtPotential$to])
         } else {
           set(dtPotential, NULL, "spreadProbRel", 1)
@@ -857,15 +862,15 @@ spread2 <- function(landscape, start = ncell(landscape) / 2 - ncol(landscape) /
     # Step 6 -- spreadProb implementation - uses an absolute probability for
     # each potential neighbour
     # Extract spreadProb for the current set of potentials
-    if (length(spreadProb) == 1) {
+    if (length(spreadProb) == 1 & !inherits(spreadProb, "SpatRaster")) {
       actualSpreadProb <- rep(spreadProb, NROW(dtPotential))
     } else {
-      actualSpreadProb <- spreadProb[dtPotential$to]
+      actualSpreadProb <- as.vector(spreadProb)[dtPotential$to]
       # remove NA values that may come from a spreadProb raster
       NAaSP <- !is.na(actualSpreadProb)
       if (any(NAaSP)) {
         if (!all(NAaSP)) {
-        dtPotential <- dtPotential[NAaSP,]
+        dtPotential <- dtPotential[NAaSP, ]
         actualSpreadProb <- actualSpreadProb[NAaSP]
       }
       }
@@ -881,10 +886,10 @@ spread2 <- function(landscape, start = ncell(landscape) / 2 - ncol(landscape) /
         asymmetry[dtPotential$to]
       }
 
-      actualAsymmetryAngle <- if (length(asymmetryAngle) == 1) {
-        asymmetryAngle
+      actualAsymmetryAngle <- if (asymmetryAngleNeedSubset) {
+        asymmetryAngle[][dtPotential$to]
       } else {
-        asymmetryAngle[dtPotential$to]
+        asymmetryAngle
       }
 
       angleQualities <- angleQuality(from = dtPotential$id, to = dtPotential$to,
@@ -901,7 +906,7 @@ spread2 <- function(landscape, start = ncell(landscape) / 2 - ncol(landscape) /
       set(dtPotential, NULL, "actualSpreadProb", actualSpreadProb)
       randoms <- runifC(length(unique(dtPotential$from)))
       dtPotential[, keep := {
-        cumProb = cumsum(actualSpreadProb)/sum(actualSpreadProb)
+        cumProb <- cumsum(actualSpreadProb) / sum(actualSpreadProb)
         draw <- randoms[.GRP]
         .I[min(which(draw <= cumProb))]},
         by = "from"]
@@ -917,13 +922,14 @@ spread2 <- function(landscape, start = ncell(landscape) / 2 - ncol(landscape) /
     # Step 8 - Remove duplicates & bind dt and dtPotential
     if (anyNAneighProbs) {
       if (isTRUE(allowOverlap > 0) | is.na(allowOverlap) | !canUseAvailable) {
-        # overlapping allowed
+        ## overlapping allowed
         dtPotential <- dtPotential[spreadProbSuccess]
         dtNROW <- NROW(dt)
         dt <- rbindlistDtDtpot(dt, dtPotential, returnFrom, needDistance, dtPotentialColNames)
 
-        # this is to prevent overlap within an event... in some cases, overlap within event is desired, so skip this block
-        if (!is.na(allowOverlap) && (any(allowOverlap %in% c(1,3) ) || isTRUE(allowOverlap))) {
+        ## this is to prevent overlap within an event...
+        ## in some cases, overlap within event is desired, so skip this block
+        if (!is.na(allowOverlap) && (any(allowOverlap %in% c(1, 3) ) || isTRUE(allowOverlap))) {
           if (identical(allowOverlap, 1) || isTRUE(allowOverlap)) {
             dt[, `:=`(dups = duplicatedInt(pixels)), by = "initialPixels"]
           } else {
@@ -940,7 +946,7 @@ spread2 <- function(landscape, start = ncell(landscape) / 2 - ncol(landscape) /
           dt <- dt[!dupes]
         }
 
-        # remove all the duplicated ones from dtPotential
+        ## remove all the duplicated ones from dtPotential
         dtPotential <- dt[-seq_len(dtNROW)]
       } else {
         # no overlapping allowed
@@ -1065,14 +1071,14 @@ spread2 <- function(landscape, start = ncell(landscape) / 2 - ncol(landscape) /
     # breaking some tests
 
     ## Extract persistenceProb for the current set of source pixels
-    if (length(persistProb) == 1) {
+    if (length(persistProb) == 1 && (!is(persistProb, "Raster") && !is(persistProb, "SpatRaster"))) {
       if (is.na(persistProb)) {
         actualPersistProb <- NULL
       } else {
         actualPersistProb <- rep(persistProb, sum(dt$state == "activeSource"))
       }
     } else {
-      actualPersistProb <- persistProb[dt[state == "activeSource", initialPixels]]
+      actualPersistProb <- persistProb[][dt[state == "activeSource", initialPixels]]
     }
 
     ## "activeSource" fires become "successful" depending on prob of persistence
@@ -1095,16 +1101,22 @@ spread2 <- function(landscape, start = ncell(landscape) / 2 - ncol(landscape) /
     #   tooSmall ==> tooSmall
     set(dt, whNotInactive, "state",
         c("inactive", "activeSource", "activeSource", "tooSmall")[
-          fmatch(activeStates, c("activeSource", "holding", "successful", "tooSmall"))])
+          fmatch2(activeStates, c("activeSource", "holding", "successful", "tooSmall"))])
 
     # Step 11 - plot it if necessary
     if (plot.it) {
+      # .requireNamespace("quickPlot")
+
       newPlot <- FALSE
       if (totalIterations == 1) {
         newPlot <- TRUE
       }
-      if (newPlot | !(exists("spread2Ras", inherits = FALSE)))
-        spread2Ras <- raster(landscape)
+      if (newPlot | !(exists("spread2Ras", inherits = FALSE))) {
+        if (any(landscapeOrigClass == "Raster"))
+          spread2Ras <- raster::raster(landscape)
+        else
+          spread2Ras <- terra::rast(landscape)
+      }
       if (returnDistances) {
         spread2Ras[dt$pixels] <- dt$distance
         newPlot <- TRUE # need to rescale legend each time
@@ -1115,7 +1127,7 @@ spread2 <- function(landscape, start = ncell(landscape) / 2 - ncol(landscape) /
         setkeyv(dt, "order")
         set(dt, NULL, "order", NULL)
       }
-      Plot(spread2Ras, new = newPlot)
+      terra::plot(spread2Ras, add = !newPlot)
     }
   } # end of main loop
 
@@ -1136,7 +1148,11 @@ spread2 <- function(landscape, start = ncell(landscape) / 2 - ncol(landscape) /
 
   # Step 13 -- return either raster or data.table
   if (asRaster) {
-    ras <- raster(landscape)
+    if (any(landscapeOrigClass == "Raster"))
+      ras <- raster::raster(landscape)
+    else
+      ras <- terra::rast(landscape)
+    # ras <- raster(landscape)
     # inside unit tests, this raster gives warnings if it is only NAs
     suppressWarnings(ras[dt$pixels] <- clusterDT[dt]$id)
     setattr(ras, "pixel", dt)
@@ -1204,11 +1220,11 @@ reorderColsWDistance <- function(needDistance, dtPotential, dtPotentialColNames)
 #' @keywords internal
 #' @rdname spread2-internals
 angleQuality <- function(from, to, landscape, actualAsymmetryAngle) {
-  from1 <- cbind(id = from, xyFromCell(landscape, from))
-  to1 <- cbind(id = from, xyFromCell(landscape, to))
+  from1 <- cbind(id = from, xyFromCell(landscape, cell = as.vector(from)))
+  to1 <- cbind(id = from, xyFromCell(landscape, cell = as.vector(to)))
   d <- .pointDirection(from = from1, to = to1)
 
-  angleQuality <- cbind(angleQuality = (cos(d[, "angles"] - rad(actualAsymmetryAngle)) + 1), d)
+  angleQuality <- cbind(angleQuality = (cos(d[, "angles"] - CircStats::rad(actualAsymmetryAngle)) + 1), d)
   angleQuality
 }
 
