@@ -1,8 +1,7 @@
-library(raster)
-library(quickPlot)
+library(terra)
 
 n <- 2
-distRas <- raster(extent(0, 40, 0, 40), res = 1)
+distRas <- rast(ext(0, 40, 0, 40), res = 1)
 coords <- cbind(x = round(runif(n, xmin(distRas), xmax(distRas))) + 0.5,
                 y = round(runif(n, xmin(distRas), xmax(distRas))) + 0.5)
 
@@ -12,22 +11,22 @@ indices <- cellFromXY(distRas, dists1[, c("x", "y")])
 invDist <- tapply(dists1[, "dists"], indices, function(x) sum(1 / (1 + x))) # idw function
 distRas[] <- as.vector(invDist)
 if (interactive()) {
-  clearPlot()
-  Plot(distRas)
+  # clearPlot()
+  terra::plot(distRas)
 }
 
 # With iterative summing via cumulativeFn to keep memory use low, with same result
 dists1 <- distanceFromEachPoint(coords[, c("x", "y"), drop = FALSE],
                                 landscape = distRas, cumulativeFn = `+`)
-idwRaster <- raster(distRas)
+idwRaster <- rast(distRas)
 idwRaster[] <- dists1[, "dists"]
-if (interactive()) Plot(idwRaster)
+if (interactive()) terra::plot(idwRaster)
 
 all(idwRaster[] == distRas[]) # TRUE
 
 # A more complex example of cumulative inverse distance sums, weighted by the value
 #  of the origin cell
-ras <- raster(extent(0, 34, 0, 34), res = 1, val = 0)
+ras <- rast(ext(0, 34, 0, 34), res = 1, val = 0)
 rp <- randomPolygons(ras, numTypes = 10) ^ 2
 n <- 15
 cells <- sample(ncell(ras), n)
@@ -39,14 +38,15 @@ dists1 <- distanceFromEachPoint(coords[, c("x", "y"), drop = FALSE],
                                 landscape = rp, distFn = distFn, cumulativeFn = `+`)
 #endCluster() # if beginCluster was run
 
-idwRaster <- raster(ras)
+idwRaster <- rast(ras)
 idwRaster[] <- dists1[, "dists"]
 if (interactive()) {
-  clearPlot()
-  Plot(rp, idwRaster)
-  sp1 <- SpatialPoints(coords)
-  Plot(sp1, addTo = "rp")
-  Plot(sp1, addTo = "idwRaster")
+  # clearPlot()
+  terra::plot(rp)
+  sp1 <- vect(coords)
+  terra::plot(sp1, add = TRUE)
+  terra::plot(idwRaster)
+  terra::plot(sp1, add = TRUE)
 }
 
 # On linux; can use numeric passed to cl; will use mclapply with mc.cores = cl
